/**
 *  Copyright (C) 2010 Daniel Francisco de Melo, Tiago Teixeira
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2010 Daniel Francisco de Melo, Tiago Teixeira
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
package gui;

import java.util.ArrayList;

public class Logica {

    private Pecas[][] tabuleiro;
    private boolean jogadorDaVez;
    private boolean podemover;
    private ArrayList<String> caminhos;
    private ArrayList<String> pecasComer;
    private String desenharDest;
    private String desenharOrig;
    private boolean diagonal[];

    public Logica(Pecas[][] tab) {
        this.tabuleiro = tab;
        this.jogadorDaVez = true;
        this.podemover = false;
        this.caminhos = new ArrayList<String>();
        this.pecasComer = new ArrayList<String>();
        desenharDest = "";
        desenharOrig = "";
        diagonal = new boolean[4];
        diagonal[0] = false;
        diagonal[1] = false;
        diagonal[2] = false;
        diagonal[3] = false;

    }

    /**
     * Recebe a origem ox/oy e o destino dx/dy que o jogador clicou
     * Compara se é Dama ou peça normal e invoca o método que irá tratar o movimento
     * Se retornar 0 pode mover a peça, se retornar diferente de 0 não pode realizar o movimento.
     *
     */
    public int mover(int ox, int oy, int dx, int dy) {
        //dx = linha e dy = coluna
        // garantindo movimentacao nas casas pretas
        if ((dx % 2 == 0) && (dy % 2 == 1) || (dx % 2 == 1) && (dy % 2 == 0)) {//testa diagonal

            if ((tabuleiro[ox][oy].isCor() == this.jogadorDaVez) && (tabuleiro[dx][dy] == null)) {//testa jogador da vez e casa vazia
                if (tabuleiro[ox][oy].getDama() == true) {//testa se a peça é dama, se for entra na rotida dela
                    if (this.dama(ox, oy, dx, dy)) {
                        return 0;
                    }
                }

                if (this.percorrer(ox, oy, dx, dy)) { //chama o percorrer, se retornar true ele entra no if, se false não entra
                    return 0;//retorna 0 e volta pro painel

                }
                if (podemover == true) {//se o podemover é true o jogador não escolheu a peça obrigatoria a ser comida
                    podemover = false;//retorna pra false
                    return 1;//return 1 ele volta pro painel mas não faz os movimentos

                } else {//se o podemover=false ele entra aqui
                    if (this.andar(ox, oy, dx, dy)) {//se percorrer = false então ele nao tem peças pra comer, ou seja ele vai só movimentar a peça
                        this.jogadorDaVez = !this.jogadorDaVez;//this.andar ele chama o metodo andar e se retornar true ele entra no if

                        return 0;
                    } else {
                        return 1;
                    }
                }
            }
        }
        return 1;
    }

    /*
     * Faz os testes para movimentar a peça
     * Define se a peça pode ser movimentada para cima ou para baixo
     * Se retornar true pode mover, antes ele testa se vira Dama.
     */
    public boolean andar(int ox, int oy, int dx, int dy) {//faz os testes pra andar
        if ((oy - dy == 1 || oy - dy == -1)) { //testa coluna proxima
            if (tabuleiro[ox][oy].isCor() == false) { // testa se é vermelho pra somar a linha
                if (ox - dx == - 1) {
                    this.testaDama(ox, oy, dx, dy);//testa se a peça movimentada vai virar dama
                    return true;
                }
            }
            if (tabuleiro[ox][oy].isCor() == true) {//testa se é amarelo pra diminuir a linha
                if (ox - dx == 1) {
                    this.testaDama(ox, oy, dx, dy);//testa se a peça movimentada vai virar dama
                    return true;
                }
            }
            return false;
        }
        return false;
    }

    /*
     * Teste pra ver se pode comer
     * Só pode comer se o movimento for para frente.
     * Se retonar true pode mover
     */
    public boolean podeComer(int lin, int col) {

        if (this.jogadorDaVez == true) {

            if ((lin - 2 >= 0) && (col - 2 >= 0)) {
                if ((lin - 2 <= 7) && (col - 2 <= 7)) {

                    if ((tabuleiro[lin - 1][col - 1] != null)) {
                        if ((tabuleiro[lin - 1][col - 1].isCor() == false)) {
                            if (tabuleiro[lin - 2][col - 2] == null) {
                                return true;

                            }
                        }
                    }
                }
            }

            if ((lin - 2 >= 0) && (col + 2 >= 0)) {
                if ((lin - 2 <= 7) && (col + 2 <= 7)) {

                    if ((tabuleiro[lin - 1][col + 1] != null)) {
                        if ((tabuleiro[lin - 1][col + 1].isCor() == false)) {
                            if (tabuleiro[lin - 2][col + 2] == null) {
                                return true;
                            }
                        }
                    }
                }
            }

        } else {

            if ((lin + 2 >= 0) && (col - 2 >= 0)) {
                if ((lin + 2 <= 7) && (col - 2 <= 7)) {

                    if ((tabuleiro[lin + 1][col - 1] != null)) {
                        if ((tabuleiro[lin + 1][col - 1].isCor() == true)) {
                            if (tabuleiro[lin + 2][col - 2] == null) {
                                return true;
                            }
                        }
                    }
                }
            }

            if ((lin + 2 >= 0) && (col + 2 >= 0)) {
                if ((lin + 2 <= 7) && (col + 2 <= 7)) {

                    if ((tabuleiro[lin + 1][col + 1] != null)) {
                        if ((tabuleiro[lin + 1][col + 1].isCor() == true)) {

                            if (tabuleiro[lin + 2][col + 2] == null) {
                                return true;
                            }
                        }
                    }
                }
            } else {
                return false;
            }
        }
        return false;
    }

    /*
     * Faz um for para percorrer todas as peças do jogador da vez
     * Se for dama ele invoca o percorreCaminhoDama, se não for dama ele invoca o percorreCaminho
     * retorna um array list com todos os caminhos para comer
     */
    public ArrayList<String> procurarTodosCaminhos() {//metodo do professor pra procurar os caminhos que pode ser comidos

        String x = "";
        for (int z = 0; z < 8; z++) {
            for (int j = 0; j < 8; j++) {
                if (tabuleiro[z][j] != null) {
                    if (tabuleiro[z][j].isCor() == jogadorDaVez) {
                        if (tabuleiro[z][j].getDama() == true) {
                            this.percorreCaminhoDama(z, j, z, j, z + "" + j, x);
                        } else {
                            if (this.podeComer(z, j)) {
                                this.percorreCaminho(z, j, z, j, z + "" + j, x);
                            }
                        }
                    }
                }
            }
        }

        diagonal[0] = false;
        diagonal[1] = false;
        diagonal[2] = false;
        diagonal[3] = false;

        // arraylist com todos os caminhos encontrados
        ArrayList<String> copia = caminhos;

        // zerando caminhos
        caminhos = new ArrayList<String>();
        return copia;

    }

    /**
     * Método recursivo para verificar o maior caminho para caputra de peças
     *
     * @param antX linha de onde a peça veio
     * @param antY coluna de onde a peça veio
     * @param xAtual linha atual da peça
     * @param yAtual coluna atual da peça
     * @param anterior todo o caminho por onde a peça passou
     */
    public void percorreCaminho(int antX, int antY, int xAtual, int yAtual, String anterior, String peca) {

        /* verificando direita acima */
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual - 2 >= 0) && (yAtual + 2 <= 7)) && ((xAtual - 2 != antX) || (yAtual + 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual - 1][yAtual + 1] != null) && (tabuleiro[xAtual - 2][yAtual + 2] == null)) {
                if ((tabuleiro[xAtual - 1][yAtual + 1].isCor() != jogadorDaVez)) {

                    //guarda as peças a serem comidas
                    String pecas = peca;
                    if (!peca.contains((xAtual - 1) + "" + (yAtual + 1))) {
                        pecas = peca + "" + (xAtual - 1) + "" + (yAtual + 1);
                        pecasComer.add(pecas);
                    }
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual + 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual - 2) + "" + (yAtual + 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual - 2, yAtual + 2, novo, pecas);
                    } else {
                        // mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual + 2));
                        System.err.println("caminho: " + anterior);
                    }
                }
            }
        }

        /* verificando direita abaixo */
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual + 2 <= 7) && (yAtual + 2 <= 7)) && ((xAtual + 2 != antX) || (yAtual + 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual + 1][yAtual + 1] != null) && (tabuleiro[xAtual + 2][yAtual + 2] == null)) {
                if ((tabuleiro[xAtual + 1][yAtual + 1].isCor() != jogadorDaVez)) {

                    //guarda as peças a serem comidas
                    String pecas = peca;
                    if (!peca.contains((xAtual + 1) + "" + (yAtual + 1))) {
                        pecas = peca + "" + (xAtual + 1) + "" + (yAtual + 1);
                        pecasComer.add(pecas);
                    }
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP

                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual + 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual + 2) + "" + (yAtual + 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual + 2, yAtual + 2, novo, pecas);
                    } else {
                        // mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual + 2));
                        System.err.println("caminho: " + anterior);
                    }
                }
            }
        }


        // esquerda acima
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual - 2 >= 0) && (yAtual - 2 >= 0)) && ((xAtual - 2 != antX) || (yAtual - 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual - 1][yAtual - 1] != null) && (tabuleiro[xAtual - 2][yAtual - 2] == null)) {
                if ((tabuleiro[xAtual - 1][yAtual - 1].isCor() != jogadorDaVez)) {

                    //guarda as peças a serem comidas
                    String pecas = peca;
                    if (!peca.contains((xAtual - 1) + "" + (yAtual - 1))) {
                        pecas = peca + "" + (xAtual - 1) + "" + (yAtual - 1);
                        pecasComer.add(pecas);
                    }
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual - 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual - 2) + "" + (yAtual - 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual - 2, yAtual - 2, novo, pecas);
                    } else {
                        //mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual - 2));
                        System.err.println("caminho: " + anterior);
                    }

                }
            }
        }



        // esquerda abaixo
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual + 2 <= 7) && (yAtual - 2 >= 0)) && ((xAtual + 2 != antX) || (yAtual - 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual + 1][yAtual - 1] != null) && (tabuleiro[xAtual + 2][yAtual - 2] == null)) {
                if ((tabuleiro[xAtual + 1][yAtual - 1].isCor() != jogadorDaVez)) {
                    //guarda as peças a serem comidas
                    String pecas = peca;
                    if (!peca.contains((xAtual + 1) + "" + (yAtual - 1))) {
                        pecas = peca + "" + (xAtual + 1) + "" + (yAtual - 1);
                        pecasComer.add(pecas);
                    }
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual - 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual + 2) + "" + (yAtual - 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual + 2, yAtual - 2, novo, pecas);
                    } else {
                        //mensagem de depuração
                        System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual - 2));
                        System.err.println("caminho: " + anterior);
                    }
                }
            }
        }
    }

    /*
     * Invoca o procurarTodosCaminhos e guarda em um ArrayList o que retornar
     * Invoca o método separarMaior que separa o maior caminho tanto das peças para comer quanto do caminho a percorrer
     * Invoca o casocomer para fazer o teste se a casa escolhida é o maior caminho
     * Se retornar true ele pode mover a peça pro lugar escolhido
     */
    public boolean percorrer(int ox, int oy, int dx, int dy) {

        ArrayList<String> caminhocomer = new ArrayList<String>();
        ArrayList<String> maior = new ArrayList<String>();
        ArrayList<String> pecasComerMaior = new ArrayList<String>();

        caminhocomer = this.procurarTodosCaminhos();

        pecasComerMaior = this.separarMaior(pecasComer);
        maior = this.separarMaior(caminhocomer);
        if (this.casocomer(ox, oy, dx, dy, maior, pecasComerMaior)) {
            return true;

        }
        return false;
    }

    /*
     * Separa o maior caminho
     * Retorna o maior caminho a ser percorrido
     */
    public ArrayList<String> separarMaior(ArrayList<String> x) {

        int maior = -1;//cria uma variavel maior pra usar dentro do for
        ArrayList<String> maiores = new ArrayList<String>();

        for (String linha : x) {

            if (linha.length() > maior) {//compara se o tamanho da string linha é maior que o for
                maiores = new ArrayList<String>();//zera a variavel maiores
                maiores.add(linha);//adiciona em uma posição do array a string linha
                maior = linha.length();//guarda o tamanho da string linha em maior

            } else if (linha.length() == maior) {//compara se o tamanho da string linha é = a maior
                maiores.add(linha);//aqui que fica guardado os maiores caminhos

            }
        }

        return maiores;
    }

    /*
     * Recebe o maior caminho e compara se o destino escolhido é igual ao maior caminho
     * Apaga as peças comidas
     * Se retornar true o destino é igual ao maior caminho
     */
    public boolean casocomer(int ox, int oy, int dx, int dy, ArrayList<String> maiores, ArrayList<String> pecasComer) {

        for (String a : maiores) {//percorre o array maiores e cada posição do array ele guarda em a

            if (a != null) {//se a!= null quer dizer que ele tem que mover a peça
                podemover = true;//seta para true a variavel que é usada para obrigar a comer
            }

            int aa = a.length();//guarda o tamanho da string em aa
            String origem = a.substring(0, 2);//separa a origem e o destino da string a
            String destino = a.substring(aa - 2, aa);

            desenharDest = desenharDest + "" + destino;
            desenharOrig = desenharOrig + "" + origem;

            String desejoOrigem = ox + "" + oy;//guarda o ox,oy,dx,dy em uma string para comparar se o que o jogador clicou é igual ao que ele tem que clicar
            String desejoDestino = dx + "" + dy;

            if (origem.equals(desejoOrigem)) {//compara se origem = desejoOrigem
                if (destino.equals(desejoDestino)) {//compara se destino = desejoDestino

                    for (int t = 0; t < maiores.size(); t++) {//compara se a peça para comer esta no caminho percorrido
                        if (a.equals(maiores.get(t))) {
                            String tmp2 = pecasComer.get(t);
                            for (int f = 0; f < tmp2.length(); f += 2) {
                                int x = Integer.parseInt(tmp2.substring(f, f + 1));
                                int y = Integer.parseInt(tmp2.substring(f + 1, f + 2));
                                this.tabuleiro[x][y] = null;//apaga a peça
                            }
                        }
                    }
                    this.testaDama(ox, oy, dx, dy);//testa se a peça movimentada vai virar dama
                    this.jogadorDaVez = !this.jogadorDaVez;//troca o jogador da vez
                    this.pecasComer = new ArrayList<String>();
                    this.desenharDest = "";
                    this.desenharOrig = "";

                    return true;//começa a retornar até ir para o classe painel
                }
            }
        }
        return false;
    }
    /*
     * Teste para ver se a peça chegou na ultima casa, se chegou vira Dama
     */

    public void testaDama(int ox, int oy, int dx, int dy) {
        if (this.jogadorDaVez == true) {

            if (dx == 0) {
                this.tabuleiro[ox][oy].setDama(true);
            }
        } else {
            if (dx == 7) {
                this.tabuleiro[ox][oy].setDama(true);
            }
        }
    }

    /*
     * se a peça Selecionada for dama entra nesse método
     * Invoca o percorrer para ver se tem peças para comer
     * Se não tiver peças para comer ele invoca o método andarDama
     * se retornar true pode mover
     */
    public boolean dama(int ox, int oy, int dx, int dy) {

        if (this.percorrer(ox, oy, dx, dy)) {
            return true;
        }
        if (podemover == true) {//se o podemover é true o jogador não escolheu a peça obrigatória a ser comida
            podemover = false;//retorna pra false
            this.pecasComer = new ArrayList<String>();
            return false;
        } else {
            if (this.andarDama(ox, oy, dx, dy)) {
                this.jogadorDaVez = !this.jogadorDaVez;//troca o jogador da vez
                this.pecasComer = new ArrayList<String>();
                return true;
            }
        }
        return false;
    }

    /*
     * Testa até onde a dama pode andar
     * Se retornar true ela pode andar ao destino escolhido
     */
    public boolean andarDama(int ox, int oy, int dx, int dy) {

        ArrayList<Integer> destdama1 = new ArrayList<Integer>();
        ArrayList<Integer> destdama2 = new ArrayList<Integer>();
        ArrayList<Integer> destdama3 = new ArrayList<Integer>();
        ArrayList<Integer> destdama4 = new ArrayList<Integer>();
        boolean diag1 = false;
        boolean diag2 = false;
        boolean diag3 = false;
        boolean diag4 = false;


        for (int k = 1; k < 7; k++) {
            if (diag1 == false) {
                if ((ox + k >= 0) && (oy + k >= 0)) {
                    if ((ox + k <= 7) && (oy + k <= 7)) {
                        if (this.tabuleiro[ox + k][oy + k] == null) {
                            destdama1.add((ox + k));
                            destdama1.add((oy + k));

                        } else {
                            diag1 = true;
                        }
                    }
                }
            }

            if (diag2 == false) {

                if ((ox - k >= 0) && (oy - k >= 0)) {
                    if ((ox - k <= 7) && (oy - k <= 7)) {
                        if (this.tabuleiro[ox - k][oy - k] == null) {
                            destdama2.add((ox - k));
                            destdama2.add((oy - k));

                        } else {
                            diag2 = true;
                        }
                    }
                }
            }

            if (diag3 == false) {

                if ((ox + k >= 0) && (oy - k >= 0)) {
                    if ((ox + k <= 7) && (oy - k <= 7)) {
                        if (this.tabuleiro[ox + k][oy - k] == null) {
                            destdama3.add((ox + k));
                            destdama3.add((oy - k));
                        } else {
                            diag3 = true;
                        }
                    }
                }
            }

            if (diag4 == false) {

                if ((ox - k >= 0) && (oy + k >= 0)) {
                    if ((ox - k <= 7) && (oy + k <= 7)) {
                        if (this.tabuleiro[ox - k][oy + k] == null) {
                            destdama4.add((ox - k));
                            destdama4.add((oy + k));

                        } else {
                            diag4 = true;
                        }
                    }
                }
            }
        }

        for (int k = 0; k < destdama1.size(); k += 2) {
            if ((dx == destdama1.get(k)) && (dy == destdama1.get(k + 1))) {
                return true;
            }
        }

        for (int k = 0; k < destdama2.size(); k += 2) {
            if ((dx == destdama2.get(k)) && (dy == destdama2.get(k + 1))) {
                return true;
            }
        }
        for (int k = 0; k < destdama3.size(); k += 2) {
            if ((dx == destdama3.get(k)) && (dy == destdama3.get(k + 1))) {
                return true;
            }
        }
        for (int k = 0; k < destdama4.size(); k += 2) {
            if ((dx == destdama4.get(k)) && (dy == destdama4.get(k + 1))) {
                return true;
            }
        }
        return false;
    }

    /*
     * retorna o destino para ser desenhado na classe painel
     */
    public String deseDest() {

        return desenharDest;
    }

    /*
     * retorna a origem para ser desenhada na classe painel
     */
    public String deseOrig() {

        return desenharOrig;
    }
    /*
     * Procura os caminhos que a dama pode comer
     * Salva os caminhos na variável caminhos e as peças para comer na variavel pecasComer
     */

    public void percorreCaminhoDama(int antX, int antY, int xAtual, int yAtual, String anterior, String peca) {

        /* verificando direita acima */
        // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
        //for para percorrer todas as diagonais
        for (int i = 1; i < 8; i++) {

            if (((xAtual - (i + 1) >= 0) && (yAtual + (i + 1) <= 7)) && ((xAtual - (i + 1) != antX) || (yAtual + (i + 1) != antY))) {
                // verificando se é possível comer
                if ((tabuleiro[xAtual - i][yAtual + i] != null) && (tabuleiro[xAtual - (i + 1)][yAtual + (i + 1)] == null)) {
                    if ((tabuleiro[xAtual - i][yAtual + i].isCor() != jogadorDaVez)) {
                        //proteção contra 2 peças juntas
                        diagonal[3] = true;
                        diagonal[0] = true;
                        if (this.peçasjuntas((xAtual - i), (yAtual + i), xAtual, yAtual)) {
                            //guarda as peças para comer
                            String pecas = peca;
                            if (!peca.contains((xAtual - i) + "" + (yAtual + i))) {
                                pecas = peca + "" + (xAtual - i) + "" + (yAtual + i);

                                pecasComer.add(pecas);
                            }
                            // verificando se esta jogada não está presente no caminho percorrido até aqui
                            // proteção contra LOOP
                            if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - (i + 1)) + "" + (yAtual + (i + 1)))) {
                                // adicionando o caminho percorrido no arraylist
                                String novo = anterior + ":" + (xAtual - (i + 1)) + "" + (yAtual + (i + 1));
                                caminhos.add(novo);
                                this.percorreCaminhoDama((xAtual - (i - 1)), (yAtual + (i - 1)), xAtual - (i + 1), yAtual + (i + 1), novo, pecas);
                            } else {
                                // mensagem de depuração
                                System.err.println("teste  : " + (xAtual - (i - 1)) + "" + (yAtual + (i - 1)) + ":" + (xAtual - (i + 1)) + "" + (yAtual + (i + 1)));
                                System.err.println("caminho: " + anterior);
                            }
                        }
                    }
                }
            }

            /* verificando direita abaixo */
            // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
            if (((xAtual + (i + 1) <= 7) && (yAtual + (i + 1) <= 7)) && ((xAtual + (i + 1) != antX) || (yAtual + (i + 1) != antY))) {
                // verificando se é possível comer
                if ((tabuleiro[xAtual + i][yAtual + i] != null) && (tabuleiro[xAtual + (i + 1)][yAtual + (i + 1)] == null)) {
                    if ((tabuleiro[xAtual + i][yAtual + i].isCor() != jogadorDaVez)) {
                        //proteção contra 2 peças juntas
                        diagonal[2] = true;
                        diagonal[1] = true;
                        if (this.peçasjuntas((xAtual + i), (yAtual + i), xAtual, yAtual)) {
                            //guarda as peças para comer
                            String pecas = peca;
                            if (!peca.contains((xAtual + i) + "" + (yAtual + i))) {
                                pecas = peca + "" + (xAtual + i) + "" + (yAtual + i);

                                pecasComer.add(pecas);
                            }
                            // verificando se esta jogada não está presente no caminho percorrido até aqui
                            // proteção contra LOOP
                            if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + (i + 1)) + "" + (yAtual + (i + 1)))) {
                                // adicionando o caminho percorrido no arraylist
                                String novo = anterior + ":" + (xAtual + (i + 1)) + "" + (yAtual + (i + 1));
                                caminhos.add(novo);
                                this.percorreCaminhoDama((xAtual + (i - 1)), (yAtual + (i - 1)), xAtual + (i + 1), yAtual + (i + 1), novo, pecas);
                            } else {
                                // mensagem de depuração
                                System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + (i + 1)) + "" + (yAtual + (i + 1)));
                                System.err.println("caminho: " + anterior);
                            }
                        }
                    }
                }
            }


            // esquerda acima
            // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
            if (((xAtual - (i + 1) >= 0) && (yAtual - (i + 1) >= 0)) && ((xAtual - (i + 1) != antX) || (yAtual - (i + 1) != antY))) {
                // verificando se é possível comer
                if ((tabuleiro[xAtual - i][yAtual - i] != null) && (tabuleiro[xAtual - (i + 1)][yAtual - (i + 1)] == null)) {
                    if ((tabuleiro[xAtual - i][yAtual - i].isCor() != jogadorDaVez)) {

                        //proteção contra 2 peças juntas
                        diagonal[1] = true;
                        diagonal[2] = true;
                        if (this.peçasjuntas((xAtual - i), (yAtual - i), xAtual, yAtual)) {
                            //guarda as peças para comer
                            String pecas = peca;
                            if (!peca.contains((xAtual - i) + "" + (yAtual - i))) {
                                pecas = peca + "" + (xAtual - i) + "" + (yAtual - i);

                                pecasComer.add(pecas);
                            }

                            // verificando se esta jogada não está presente no caminho percorrido até aqui
                            // proteção contra LOOP
                            if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - (i + 1)) + "" + (yAtual - (i + 1)))) {
                                // adicionando o caminho percorrido no arraylist
                                String novo = anterior + ":" + (xAtual - (i + 1)) + "" + (yAtual - (i + 1));
                                caminhos.add(novo);
                                this.percorreCaminhoDama((xAtual - (i - 1)), (yAtual - (i - 1)), xAtual - (i + 1), yAtual - (i + 1), novo, pecas);
                            } else {
                                //mensagem de depuração
                                System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - (i + 1)) + "" + (yAtual - (i + 1)));
                                System.err.println("caminho: " + anterior);
                            }
                        }
                    }
                }
            }



            // esquerda abaixo
            // garantindo que não ocorreu estouro dos limites da matriz e que não volte pela casa de onde veio
            if (((xAtual + (i + 1) <= 7) && (yAtual - (i + 1) >= 0)) && ((xAtual + (i + 1) != antX) || (yAtual - (i + 1) != antY))) {
                // verificando se é possível comer
                if ((tabuleiro[xAtual + i][yAtual - i] != null) && (tabuleiro[xAtual + (i + 1)][yAtual - (i + 1)] == null)) {
                    if ((tabuleiro[xAtual + i][yAtual - i].isCor() != jogadorDaVez)) {

                        //proteção contra 2 peças juntas
                        diagonal[3] = true;
                        diagonal[0] = true;
                        if (this.peçasjuntas((xAtual + i), (yAtual - i), xAtual, yAtual)) {

                            //guarda as peças para comer
                            String pecas = peca;
                            if (!peca.contains((xAtual + i) + "" + (yAtual - i))) {
                                pecas = peca + "" + (xAtual + i) + "" + (yAtual - i);

                                pecasComer.add(pecas);
                            }
                            // verificando se esta jogada não está presente no caminho percorrido até aqui
                            // proteção contra LOOP
                            if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + (i + 1)) + "" + (yAtual - (i + 1)))) {
                                // adicionando o caminho percorrido no arraylist
                                String novo = anterior + ":" + (xAtual + (i + 1)) + "" + (yAtual - (i + 1));
                                caminhos.add(novo);
                                this.percorreCaminhoDama((xAtual + (i - 1)), (yAtual - (i - 1)), xAtual + (i + 1), yAtual - (i + 1), novo, pecas);
                            } else {
                                //mensagem de depuração
                                System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + (i + 1)) + "" + (yAtual - (i + 1)));
                                System.err.println("caminho: " + anterior);
                            }
                        }
                    }
                }
            }
        }
    }

    /*
     * Verifica se não existe 2 peças ou mais juntas na mesma diagonal
     * Se retornar true não existe
     */
    public boolean peçasjuntas(int x, int y, int p, int q) {

        if (diagonal[3] == true) {
            if ((((x + 2) <= 7) && ((y - 2) >= 0))) {
                if ((this.tabuleiro[x + 1][y - 1] != null) && (this.tabuleiro[x + 1][y - 1].isCor() != jogadorDaVez)) {
                    if (((x + 1) != p) || (y - 1) != q) {
                        diagonal[0] = false;
                        diagonal[3] = false;
                        return false;

                    }
                }
            }
        }
        if (diagonal[2] == true) {
            if ((((x - 2) >= 0) && ((y - 2) >= 0))) {
                if ((this.tabuleiro[x - 1][y - 1] != null) && (this.tabuleiro[x - 1][y - 1].isCor() != jogadorDaVez)) {
                    if (((x - 1) != p) || (y - 1) != q) {
                        diagonal[2] = false;
                        diagonal[1] = false;
                        return false;
                    }
                }
            }
        }
        if (diagonal[1] == true) {
            if ((((x + 2) <= 7) && ((y + 2) <= 7))) {
                if ((this.tabuleiro[x + 1][y + 1] != null) && (this.tabuleiro[x + 1][y + 1].isCor() != jogadorDaVez)) {
                    if (((x + 1) != p) || (y + 1) != q) {
                        diagonal[1] = false;
                        diagonal[2] = false;
                        return false;
                    }
                }
            }
        }
        if (diagonal[0] == true) {
            if ((((x - 2) >= 0) && ((y + 2) <= 7))) {
                if ((this.tabuleiro[x - 1][y + 1] != null) && (this.tabuleiro[x - 1][y + 1].isCor() != jogadorDaVez)) {
                    if (((x - 1) != p) || (y + 1) != q) {
                        diagonal[0] = false;
                        diagonal[3] = false;
                        return false;
                    }
                }
            }
        }
        diagonal[0] = false;
        diagonal[3] = false;
        diagonal[1] = false;
        diagonal[2] = false;

        return true;
    }
}
