/**
 *  Copyright (C) 2010 Daniel Francisco de Melo, Tiago Teixeira
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2010 Daniel Francisco de Melo, Tiago Teixeira
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */

package servidor;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.ArrayList;
import protocolo.Mensagem;


public class ThreadServidor extends Thread {

    private Socket conexao;
    private ArrayList<ObjectOutputStream> clientes;

    public ThreadServidor(Socket c, ArrayList<ObjectOutputStream> cli) {
        this.conexao = c;
        this.clientes = cli;

    }

    @Override
    public void run() {
        ObjectInputStream fluxoEntrada = null;
        ObjectOutputStream fluxoSaida = null;
        String nomeDoCliente = "";
        try {
            // Objetos serão transportados pelo sockets
            fluxoEntrada = new ObjectInputStream(conexao.getInputStream());
            fluxoSaida = new ObjectOutputStream(conexao.getOutputStream());

            Mensagem recebida = null;

            //adicionado o fluxo de saida do cliente no arraylist
            //isso permite ao servidor enviar mensagens para todos os clientes conectados
            clientes.add(fluxoSaida);

            //esperando o cliente enviar seu nome.
            //mensagem com código 15
            recebida = (Mensagem) fluxoEntrada.readObject();

            // se a conexao do cliente cair, encerre a thread
            if (recebida == null) {
                return;
            }

            if (recebida.codigo == 15) {
                nomeDoCliente = recebida.texto;
                System.out.println(nomeDoCliente + " conectou...");
            }

            // processando as mensagem recebidas do cliente
            // mensagem com codigo 10 indica fim de conexao
            do {
                recebida = (Mensagem) fluxoEntrada.readObject();
                // codigo da mensagem = 100 indica que é mensagem de batepapo
                // imprima a mensagem no console do servidor e envie essa mensagem
                // para os demais clientes conectados ao servidor
                if (recebida.codigo == 100) {
                    System.out.println(nomeDoCliente + ": " + recebida.texto);
                    for (ObjectOutputStream saidaCli : clientes) {
                        // evitando enviar a mensagem para o cliente que a originou
                        if (saidaCli != fluxoSaida) {
                            //incluindo o nome do cliente na mensagem de batepapo
                            recebida.texto = nomeDoCliente + ": " + recebida.texto;
                            saidaCli.writeObject(recebida);
                            saidaCli.flush();
                            saidaCli.reset();
                        }
                    }

                }


            } while (recebida.codigo != 10);
        } catch (EOFException eof) {
            System.out.println(nomeDoCliente + " saiu...");
        } catch (IOException ex) {
            System.err.println(ex.toString());
        } catch (Exception e) {
            System.err.println(e.toString());
        } finally {
            try {
                // removendo fluxo de saida do arraylist, afinal o cliente desconectou
                clientes.remove(fluxoSaida);

                // fechando fluxos
                fluxoEntrada.close();
                fluxoSaida.close();
                conexao.close();

            } catch (IOException ex) {
                System.err.println("finally: " + ex.toString());
            }
        }
    }
}
